﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading;
using System.Windows.Forms;
using Sunweaver.Parsing;
using Sunweaver;
using ScintillaNET;

namespace IDEControls
{
    public partial class DevControl : UserControl
    {
        public const double TIME_AFTER_TYPING = 500;
        public const int ERROR_STYLE = 2;
        System.Timers.Timer timer;
        public DevControl()
        {
            InitializeComponent();
            //make sure line numbers are visible
            scintilla.Margins[0].Width = 20;
            timer = new System.Timers.Timer(TIME_AFTER_TYPING);
            timer.Elapsed += new System.Timers.ElapsedEventHandler(updateStyle_timer);
            defineLanguage();
        }

        private void defineLanguage()
        {

            StringBuilder commandsSB = new StringBuilder();

            foreach (KeyValuePair<string, Type> command in Sunweaver.DNASystem.Commands)
            {
                commandsSB.Append(command.Key);
                commandsSB.Append(" ");
            }


            StringBuilder sysvarSB = new StringBuilder();
            foreach (KeyValuePair<string, short> sysvar in Sunweaver.DNASystem.Sysvars)
            {
                sysvarSB.Append(sysvar.Key);
                sysvarSB.Append(" ");
            }

            scintilla.Lexing.Lexer = ScintillaNET.Lexer.Cpp;
            scintilla.Lexing.LineCommentPrefix = "//";
            scintilla.Lexing.Keywords[0] = commandsSB.ToString();
            scintilla.Lexing.Keywords[1] = sysvarSB.ToString();

            Font mainFont = new System.Drawing.Font("Courier New", 10F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((byte)(0)));

            scintilla.Styles.Default.Font = mainFont;
            // this isn't the hack you are looking for ~~~~#\
            for (int i = 0; i < 1000; i++)
            {
                scintilla.Styles[i].Font = mainFont;
            }
            scintilla.Styles[scintilla.Lexing.StyleNameMap["WORD"]].ForeColor = System.Drawing.Color.Blue;
            scintilla.Styles[scintilla.Lexing.StyleNameMap["WORD2"]].ForeColor = System.Drawing.Color.Turquoise;
        }

        #region events

        private void scintilla_KeyUp(object sender, KeyEventArgs e)
        {
            timer.Stop();
            timer.Start();
        }

        private void updateStyle_timer(object source, System.Timers.ElapsedEventArgs e)
        {
            timer.Stop();
            if (syntaxBackgroundWorker.IsBusy != true)
            {
                syntaxBackgroundWorker.RunWorkerAsync();
            }
        }
        private void syntaxBackgroundWorker_DoWork(object sender, DoWorkEventArgs e)
        {
            doSyntaxUpdate();
        }

        #endregion

        #region Syntax Errors and Highlighting

        public void doSyntaxUpdate()
        {
            List<CompilationMessage> compilationErrors;
            Chromosome cms = new Chromosome();
            Parser.ParseDNA(GetCode(), ref cms, out compilationErrors);
            clearErrorMessages();
            ClearErrorIndicators();
            foreach (CompilationMessage cm in compilationErrors)
            {
                AddErrorMessage(cm);
                AddErrorIndicator(cm);
            }
        }

        #region Get code delegation

        public delegate string GetCodeDelegator();

        public string GetCode()
        {
            return InvokeGetCodeThreaded();
        }

        public string InvokeGetCodeThreaded()
        {
            if (scintilla.InvokeRequired)
            {
                return (string)scintilla.Invoke(new GetCodeDelegator(GetCodeThreaded));
            }
            return GetCodeThreaded();
        }

        private string GetCodeThreaded()
        {
            return scintilla.Text;
        }

        #endregion

        #region ADD     - error message
        public delegate void AddErrorMessageDelegator(CompilationMessage cm);

        public void AddErrorMessage(CompilationMessage cm)
        {
            InvokeAddErrorMessageThreaded(cm);
        }

        public void InvokeAddErrorMessageThreaded(CompilationMessage cm)
        {
            if (compilationMessagesList.InvokeRequired)
            {
                compilationMessagesList.Invoke(new AddErrorMessageDelegator(AddErrorMessageThreaded), new object[] { cm });
            }
            else
            {
                AddErrorMessageThreaded(cm);
            }
        }

        private void AddErrorMessageThreaded(CompilationMessage cm)
        {
            ListViewItem item = null;
            if (cm.message.Contains("__anon"))
            {
                string splitString = cm.message.Split(new string[] { "__anon" }, StringSplitOptions.None)[0];
                item = new ListViewItem(string.Concat(splitString, "{"));
            }
            else
            {
                item = new ListViewItem(cm.message);
            }
            item.SubItems.Add(cm.token.line.ToString());
            item.SubItems.Add(cm.token.column.ToString());
            compilationMessagesList.Items.Add(item);
        }
        #endregion

        #region CLEAR   - error message
        public delegate void ClearErrorMessagesDelegator();

        public void clearErrorMessages()
        {
            InvokeClearErrorMessagesThreaded();
        }

        public void InvokeClearErrorMessagesThreaded()
        {
            if (compilationMessagesList.InvokeRequired)
            {
                compilationMessagesList.Invoke(new ClearErrorMessagesDelegator(ClearErrorMessagesThreaded));
            }
            else
            {
                ClearErrorMessagesThreaded();
            }
        }

        private void ClearErrorMessagesThreaded()
        {
            compilationMessagesList.Items.Clear();
        }
        #endregion

        #region ADD     - error message indicators
        public delegate void AddErrorIndicatorDelegator(CompilationMessage cm);

        public void AddErrorIndicator(CompilationMessage cm)
        {
            InvokeAddErrorIndicatorThreaded(cm);
        }

        public void InvokeAddErrorIndicatorThreaded(CompilationMessage cm)
        {
            if (scintilla.InvokeRequired)
            {
                scintilla.Invoke(new AddErrorIndicatorDelegator(AddErrorIndicatorThreaded), new object[] { cm });
            }
            else
            {
                AddErrorIndicatorThreaded(cm);
            }
        }

        private void AddErrorIndicatorThreaded(CompilationMessage cm)
        {
            string tokenString = cm.token.token;
            int start = getPosition(cm.token.line, cm.token.column);
            ScintillaNET.Range rangeIndicator = null;
            if (tokenString.Contains("__anon"))
            {
                string splitString = tokenString.Split(new string[] { "__anon" }, StringSplitOptions.None)[0];
                Console.WriteLine(splitString);
                rangeIndicator = new ScintillaNET.Range(start, start + splitString.Length, scintilla);
            }
            else
            {
                rangeIndicator = new ScintillaNET.Range(start, start + tokenString.Length, scintilla);
            }
            rangeIndicator.SetIndicator((int)ScintillaNET.IndicatorStyle.Squiggle);

        }
        #endregion

        #region CLEAR   - error message indicators
        public delegate void ClearErrorIndicatorsDelegator();

        public void ClearErrorIndicators()
        {
            InvokeClearErrorIndicatorsThreaded();
        }

        public void InvokeClearErrorIndicatorsThreaded()
        {
            if (scintilla.InvokeRequired)
            {
                scintilla.Invoke(new ClearErrorIndicatorsDelegator(ClearErrorIndicatorsThreaded));
            }
            else
            {
                ClearErrorIndicatorsThreaded();
            }
        }

        private void ClearErrorIndicatorsThreaded()
        {
            scintilla.GetRange().ClearIndicator((int)ScintillaNET.IndicatorStyle.Squiggle);
        }

        #endregion
        #endregion //syntax errors

        #region Get code delegation

        public delegate void SetTextDelegator(string code);
        
        public void SetText(string code)
        {
            InvokeSetTextThreaded(code);
            doSyntaxUpdate();
        }

        public void InvokeSetTextThreaded(string code)
        {
            if (scintilla.InvokeRequired)
            {
                scintilla.Invoke(new SetTextDelegator(SetTextThreaded), new object[] {code});
            }
            SetTextThreaded(code);
        }

        private void SetTextThreaded(string code)
        {
            scintilla.Text = code;
        }

        #endregion

        #region undo/redo
        public Boolean canUndo()
        {
            return scintilla.UndoRedo.CanUndo;
        }

        public Boolean canRedo()
        {
            return scintilla.UndoRedo.CanRedo;
        }
        public void undo()
        {
            scintilla.UndoRedo.Undo();
        }

        public void redo()
        {
            scintilla.UndoRedo.Redo();
        }
        #endregion

        private void compilationMessagesList_ItemActivate(object sender, EventArgs e)
        {
            int line = Convert.ToInt32(compilationMessagesList.SelectedItems[0].SubItems[1].Text);
            int column = Convert.ToInt32(compilationMessagesList.SelectedItems[0].SubItems[2].Text);
            scintilla.Caret.Goto(getPosition(line, column));
            scintilla.Focus();
        }

        private int getPosition(int line, int column)
        {
            return scintilla.Lines[line - 1].StartPosition + column - 1;
        }

        public void showFind()
        {
            scintilla.FindReplace.ShowFind();
        }

        public void showFindAndReplace()
        {
            scintilla.FindReplace.ShowReplace();
        }

    }
}
