﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading;
using System.Windows.Forms;
using Sunweaver.Parsing;
using Sunweaver;
using ScintillaNET;
using System.IO;
using IDEControls;

namespace IDE
{
    public partial class IDEControl : UserControl
    {
        Form parent;
        public IDEControl(Form parent)
        {
            this.parent = parent;
            InitializeComponent();
        }

        #region file stuff
        private void newToolStripMenuItem_Click(object sender, EventArgs e)
        {
            tabsWindow.newTab();
        }

        private void openToolStripMenuItem_Click(object sender, EventArgs e)
        {
            OpenFileDialog ofd = new OpenFileDialog();
            ofd.ShowDialog(this);
            ofd.CheckFileExists = true;
            ofd.Multiselect = false;
            string filename = ofd.FileName;
            if(File.Exists(filename))
                tabsWindow.loadFileTab(ofd.FileName);
        }

        private void saveToolStripMenuItem_Click(object sender, EventArgs e)
        {
            CodingTabPage ctp = getCurrentlySelectedTab();
            string filename = ctp.Filename;
            if (filename != null)
                tabsWindow.saveFileCurrentTab(ctp, filename);
            else
                saveAs(ctp);
        }

        private void saveAsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            CodingTabPage ctp = getCurrentlySelectedTab();
            saveAs(ctp);
        }

        private void saveAs(CodingTabPage ctp)
        {
            string filename;
            SaveFileDialog sfd = new SaveFileDialog();
            sfd.ShowDialog(this);
            sfd.OverwritePrompt = true;
            if (sfd.FileName != "")
            {
                filename = sfd.FileName;
                tabsWindow.saveFileCurrentTab(ctp, filename);
            }
        }

        private void closeTabToolStripMenuItem_Click(object sender, EventArgs e)
        {
            this.tabsWindow.CloseCurrentTab();
        }

        private void exitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            parent.Close();
        }

        #endregion

        private void undoToolStripMenuItem_Click(object sender, EventArgs e)
        {
            tabsWindow.getCurrentlySelectedTab().getDevControl().undo();
        }

        private void redoToolStripMenuItem_Click(object sender, EventArgs e)
        {
            tabsWindow.getCurrentlySelectedTab().getDevControl().redo();
        }

        private void editToolStripMenuItem_DropDown(object sender, EventArgs e)
        {
            undoToolStripMenuItem.Enabled = tabsWindow.getCurrentlySelectedTab().getDevControl().canUndo();
            redoToolStripMenuItem.Enabled = tabsWindow.getCurrentlySelectedTab().getDevControl().canRedo();
        }
        private CodingTabPage getCurrentlySelectedTab()
        {
            int selectedTab = tabsWindow.GetTabs().SelectedIndex;
            CodingTabPage ctp = (CodingTabPage)tabsWindow.GetTabs().Controls[selectedTab];
            return ctp;
        }

        private void findToolStripMenuItem_Click(object sender, EventArgs e)
        {
            this.getCurrentlySelectedTab().getDevControl().showFind();
        }

        private void findAndReplaceToolStripMenuItem_Click(object sender, EventArgs e)
        {
            this.getCurrentlySelectedTab().getDevControl().showFindAndReplace();
        }

    }
}